@@ -1,6 +1,6 @@
 /*
  * ansilove.c
- * Ansilove 4.0.2
+ * Ansilove 4.0.6
  * https://www.ansilove.org
  *
  * Copyright (c) 2011-2019 Stefan Vogt, Brian Cassidy, and Frederic Cambus
@@ -20,14 +20,16 @@
 #include <stdlib.h>
 #include <string.h>
 
-#ifndef HAVE_PLEDGE
-#include "pledge.h"
-#endif
-
-#ifndef HAVE_STRTONUM
-#include "strtonum.h"
+#ifdef HAVE_SECCOMP
+#include <sys/prctl.h>
+#include <sys/syscall.h>
+#include <linux/audit.h>
+#include <linux/filter.h>
+#include <linux/seccomp.h>
+#include "seccomp.h"
 #endif
 
+#include "compat.h"
 #include "config.h"
 #include "fonts.h"
 #include "sauce.h"
@@ -35,23 +37,29 @@
 
 /* prototypes */
 static void synopsis(void);
+static void version(void);
 
-/* following the IEEE Std 1003.1 for utility conventions */
 static void
 synopsis(void)
 {
-	fprintf(stderr, "\nSYNOPSIS\n"
-	    "     ansilove [-dhirsv] [-b bits] [-c columns] [-f font] [-m mode]"
-	    " [-o file]\n"
-	    "              [-R factor] file\n\n");
+	fprintf(stdout, "SYNOPSIS\n"
+	    "     ansilove [-dhiqrsv] [-b bits] [-c columns] [-f font]"
+	    " [-m mode] [-o file]\n"
+	    "              [-R factor] file\n");
+}
+
+static void
+version(void)
+{
+	fprintf(stdout, "AnsiLove/C %s - ANSI / ASCII art to PNG converter\n"
+	    "Copyright (c) 2011-2019 Stefan Vogt, Brian Cassidy, and "
+	    "Frederic Cambus.\n", VERSION);
 }
 
 int
 main(int argc, char *argv[])
 {
-	fprintf(stderr, "AnsiLove/C %s - ANSI / ASCII art to PNG converter\n"
-	    "Copyright (c) 2011-2019 Stefan Vogt, Brian Cassidy, and "
-	    "Frederic Cambus.\n", VERSION);
+	FILE *messages = NULL;
 
 	/* SAUCE record related bool types */
 	bool justDisplaySAUCE = false;
@@ -74,35 +82,40 @@ main(int argc, char *argv[])
 
 	const char *errstr;
 
-	if (ansilove_init(&ctx, &options) == -1) {
-		fprintf(stderr, "\n%s\n", ansilove_error(&ctx));
+	if (pledge("stdio cpath rpath wpath", NULL) == -1)
+		err(EXIT_FAILURE, "pledge");
+
+#ifdef HAVE_SECCOMP
+	if (prctl(PR_SET_NO_NEW_PRIVS, 1, 0, 0, 0)) {
+		perror("Can't initialize seccomp");
 		return EXIT_FAILURE;
 	}
 
-	if (pledge("stdio cpath rpath wpath", NULL) == -1) {
-		err(EXIT_FAILURE, "pledge");
+	if (prctl(PR_SET_SECCOMP, SECCOMP_MODE_FILTER, &ansilove)) {
+		perror("Can't load seccomp filter");
+		return EXIT_FAILURE;
 	}
+#endif
 
-	while ((getoptFlag = getopt(argc, argv, "b:c:df:him:o:rR:sv")) != -1) {
+	if (ansilove_init(&ctx, &options) == -1)
+		errx(EXIT_FAILURE, "%s", ansilove_error(&ctx));
+
+	while ((getoptFlag = getopt(argc, argv, "b:c:df:him:o:qrR:sv")) != -1) {
 		switch (getoptFlag) {
 		case 'b':
 			/* convert numeric command line flags to integer values */
 			options.bits = strtonum(optarg, 8, 9, &errstr);
 
-			if (errstr) {
-				fprintf(stderr, "\nInvalid value for bits (must be 8 or 9).\n\n");
-				return EXIT_FAILURE;
-			}
+			if (errstr)
+				errx(EXIT_FAILURE, "Invalid value for bits (must be 8 or 9).");
 
 			break;
 		case 'c':
 			/* convert numeric command line flags to integer values */
 			options.columns = strtonum(optarg, 1, 4096, &errstr);
 
-			if (errstr) {
-				fprintf(stderr, "\nInvalid value for columns (must range from 1 to 4096).\n\n");
-				return EXIT_FAILURE;
-			}
+			if (errstr)
+				errx(EXIT_FAILURE, "\nInvalid value for columns (must range from 1 to 4096).");
 
 			break;
 		case 'd':
@@ -135,6 +148,9 @@ main(int argc, char *argv[])
 		case 'o':
 			output = optarg;
 			break;
+		case 'q':
+			messages = fopen("/dev/null", "w");
+			break;
 		case 'r':
 			options.scale_factor = 2;
 			break;
@@ -142,16 +158,15 @@ main(int argc, char *argv[])
 			/* convert numeric command line flags to integer values */
 			options.scale_factor = strtonum(optarg, 2, 8, &errstr);
 
-			if (errstr) {
-				fprintf(stderr, "\nInvalid value for retina scale factor (must range from 2 to 8).\n\n");
-				return EXIT_FAILURE;
-			}
+			if (errstr)
+				errx(EXIT_FAILURE, "Invalid value for retina scale factor (must range from 2 to 8).");
 
 			break;
 		case 's':
 			justDisplaySAUCE = true;
 			break;
 		case 'v':
+			version();
 			return EXIT_SUCCESS;
 		}
 	}
@@ -166,13 +181,16 @@ main(int argc, char *argv[])
 	argc -= optind;
 	argv += optind;
 
+	/* if -q flag was not set, default to stdout */
+	if (!messages)
+		messages = stdout;
+
 	/* let's check the file for a valid SAUCE record */
 	sauce *record = sauceReadFileName(input);
 
 	/* record == NULL also means there is no file, we can stop here */
 	if (record == NULL) {
-		fprintf(stderr, "\nFile %s not found.\n\n", input);
-		return EXIT_FAILURE;
+		errx(EXIT_FAILURE, "File %s not found.", input);
 	} else {
 		/* if we find a SAUCE record, update bool flag */
 		if (!strcmp(record->ID, SAUCE_ID)) {
@@ -184,36 +202,30 @@ main(int argc, char *argv[])
 		/* create output file name if output is not specified */
 		if (!output) {
 			/* appending ".png" extension to output file name */
-			if (asprintf(&fileName, "%s%s", input, ".png") == -1) {
-				fprintf(stderr, "Memory allocation error.\n\n");
-				return EXIT_FAILURE;
-			}
+			if (asprintf(&fileName, "%s%s", input, ".png") == -1)
+				errx(EXIT_FAILURE, "Memory allocation error.");
 		} else {
-			fileName = output;
+			fileName = strdup(output);
 		}
 
 		/* display name of input and output files */
-		fprintf(stderr, "\nInput File: %s\n", input);
-		fprintf(stderr, "Output File: %s\n", fileName);
+		fprintf(messages, "Input File: %s\n", input);
+		fprintf(messages, "Output File: %s\n", fileName);
 
 		/* get file extension */
 		char *fext = strrchr(input, '.');
-		fext = fext ? strtolower(strdup(++fext)) : "";
+		fext = fext ? strtolower(strdup(++fext)) : strdup("");
 
 		/* check if current file has a .diz extension */
 		if (!strcmp(fext, "diz"))
 			options.diz = true;
 
-		if (ansilove_loadfile(&ctx, input) == -1) {
-			fprintf(stderr, "\n%s\n", ansilove_error(&ctx));
-			return EXIT_FAILURE;
-		}
+		if (ansilove_loadfile(&ctx, input) == -1)
+			errx(EXIT_FAILURE, "%s", ansilove_error(&ctx));
 
 		/* adjust the file size if file contains a SAUCE record */
-		if (fileHasSAUCE) {
-			sauce *saucerec = sauceReadFileName(input);
-			ctx.length -= 129 - (saucerec->comments > 0 ? 5 + 64 * saucerec->comments : 0);
-		}
+		if (fileHasSAUCE)
+			ctx.length -= 129 - (record->comments > 0 ? 5 + 64 * record->comments : 0);
 
 		int (*loader)(struct ansilove_ctx *, struct ansilove_options *);
 
@@ -238,71 +250,73 @@ main(int argc, char *argv[])
 			fileIsANSi = true;
 		}
 
-		if (loader(&ctx, &options) == -1) {
-			fprintf(stderr, "\n%s\n", ansilove_error(&ctx));
-			return EXIT_FAILURE;
-		}
+		if (loader(&ctx, &options) == -1)
+			errx(EXIT_FAILURE, "%s", ansilove_error(&ctx));
 
 		/* create the output file */
-		if (ansilove_savefile(&ctx, fileName) == -1) {
-			fprintf(stderr, "\n%s\n", ansilove_error(&ctx));
-			return EXIT_FAILURE;
-		}
+		if (ansilove_savefile(&ctx, fileName) == -1)
+			errx(EXIT_FAILURE, "%s", ansilove_error(&ctx));
 
 		/* gather information and report to the command line */
 		if (fileIsANSi || fileIsBinary ||
 		    fileIsPCBoard || fileIsTundra) {
-			fprintf(stderr, "Font: %s\n", font ? font : "80x25");
+			fprintf(messages, "Font: %s\n", font ? font : "80x25");
 
-			fprintf(stderr, "Bits: %d\n", options.bits);
+			fprintf(messages, "Bits: %d\n", options.bits);
 		}
 
 		if (options.icecolors && (fileIsANSi || fileIsBinary))
-			fprintf(stderr, "iCE Colors: enabled\n");
+			fprintf(messages, "iCE Colors: enabled\n");
 
 		if (fileIsANSi || fileIsBinary || fileIsTundra)
-			fprintf(stderr, "Columns: %d\n", options.columns);
+			fprintf(messages, "Columns: %d\n", options.columns);
 
 		if (options.scale_factor)
-			fprintf(stderr, "Scale factor: %d\n", options.scale_factor);
+			fprintf(messages, "Scale factor: %d\n", options.scale_factor);
+
+		free(fileName);
+		free(fext);
 	}
 
 	/* either display SAUCE or tell us if there is no record */
 	if (!fileHasSAUCE) {
-		fprintf(stderr, "\nFile %s does not have a SAUCE record.\n", input);
+		fprintf(messages, "\nFile %s does not have a SAUCE record.\n", input);
 	} else {
-		fprintf(stderr, "\nId: %s v%s\n", record->ID, record->version);
-		fprintf(stderr, "Title: %s\n", record->title);
-		fprintf(stderr, "Author: %s\n", record->author);
-		fprintf(stderr, "Group: %s\n", record->group);
-		fprintf(stderr, "Date: %s\n", record->date);
-		fprintf(stderr, "Datatype: %d\n", record->dataType);
-		fprintf(stderr, "Filetype: %d\n", record->fileType);
+		fprintf(messages, "\nId: %s v%s\n", record->ID, record->version);
+		fprintf(messages, "Title: %s\n", record->title);
+		fprintf(messages, "Author: %s\n", record->author);
+		fprintf(messages, "Group: %s\n", record->group);
+		fprintf(messages, "Date: %s\n", record->date);
+		fprintf(messages, "Datatype: %d\n", record->dataType);
+		fprintf(messages, "Filetype: %d\n", record->fileType);
 		if (record->flags) {
-			fprintf(stderr, "Flags: %d\n", record->flags);
+			fprintf(messages, "Flags: %d\n", record->flags);
 		}
 		if (record->tinfo1) {
-			fprintf(stderr, "Tinfo1: %d\n", record->tinfo1);
+			fprintf(messages, "Tinfo1: %d\n", record->tinfo1);
 		}
 		if (record->tinfo2) {
-			fprintf(stderr, "Tinfo2: %d\n", record->tinfo2);
+			fprintf(messages, "Tinfo2: %d\n", record->tinfo2);
 		}
 		if (record->tinfo3) {
-			fprintf(stderr, "Tinfo3: %d\n", record->tinfo3);
+			fprintf(messages, "Tinfo3: %d\n", record->tinfo3);
 		}
 		if (record->tinfo4) {
-			fprintf(stderr, "Tinfo4: %d\n", record->tinfo4);
+			fprintf(messages, "Tinfo4: %d\n", record->tinfo4);
 		}
-		fprintf(stderr, "Tinfos: %s\n", record->tinfos);
+		fprintf(messages, "Tinfos: %s\n", record->tinfos);
 		if (record->comments > 0) {
-			fprintf(stderr, "Comments: ");
+			fprintf(messages, "Comments: ");
 			for (int32_t i = 0; i < record->comments; i++) {
-				fprintf(stderr, "%s\n", record->comment_lines[i]);
+				fprintf(messages, "%s\n", record->comment_lines[i]);
 			}
 		}
 	}
 
 	ansilove_clean(&ctx);
 
+	free(record->comment_lines);
+	free(record);
+
 	return EXIT_SUCCESS;
 }
