@@ -1,6 +1,6 @@
 /*
  * sauce.c
- * Ansilove 4.0.2
+ * Ansilove 4.0.6
  * https://www.ansilove.org
  *
  * Copyright (c) 2011-2019 Stefan Vogt, Brian Cassidy, and Frederic Cambus
@@ -38,6 +38,7 @@ sauceReadFile(FILE *file)
 {
 	sauce *record;
 	record = malloc(sizeof *record);
+	memset(record, 0, sizeof *record);
 
 	if (record != NULL) {
 		readRecord(file, record);
@@ -49,7 +50,6 @@ void
 readRecord(FILE *file, sauce *record)
 {
 	if (fseek(file, 0 - RECORD_SIZE, SEEK_END) != EXIT_SUCCESS) {
-		free(record);
 		return;
 	}
 
@@ -57,7 +57,6 @@ readRecord(FILE *file, sauce *record)
 	record->ID[sizeof (record->ID) - 1] = '\0';
 
 	if (read_status != 1 || strcmp(record->ID, SAUCE_ID) != 0) {
-		free(record);
 		return;
 	}
 	fread(record->version, sizeof (record->version) - 1, 1, file);
@@ -83,7 +82,6 @@ readRecord(FILE *file, sauce *record)
 	record->tinfos[sizeof (record->tinfos) - 1] = '\0';
 
 	if (ferror(file) != EXIT_SUCCESS) {
-		free(record);
 		return;
 	}
 
@@ -94,9 +92,6 @@ readRecord(FILE *file, sauce *record)
 			if (readComments(file, record->comment_lines, record->comments) == -1) {
 				record->comments = 0;
 			}
-		} else {
-			free(record);
-			return;
 		}
 	}
 }
@@ -112,7 +107,6 @@ readComments(FILE *file, char **comment_lines, int32_t comments)
 		ID[sizeof (ID) - 1] = '\0';
 
 		if (strcmp(ID, COMMENT_ID) != 0) {
-			free(comment_lines);
 			return -1;
 		}
 
@@ -125,16 +119,13 @@ readComments(FILE *file, char **comment_lines, int32_t comments)
 			if (ferror(file) == EXIT_SUCCESS) {
 				comment_lines[i] = strdup(buf);
 				if (comment_lines[i] == NULL) {
-					free(comment_lines);
 					return -1;
 				}
 			} else {
-				free(comment_lines);
 				return -1;
 			}
 		}
 		return 0;
 	}
-	free(comment_lines);
 	return -1;
 }
